<?php
//  Copyright (c) 2014 Daniel Robert Bradley. All rights reserved.
//  This software is distributed under the terms of the GNU Lesser General Public License version 2.1
?>
<?php

class JSON4
{
    static function EncodeResults( $indent, $results )
    {
        $json = "";
    
        $json .= "{";
        
        if ( is_array( $results ) && (0 < count( $results ) ) )
        {
            $json .= '"results":';
            $json .= "[";
            
            $sep = "";
            
            foreach ( $results as $tuple )
            {
                $json .= $sep;
                $json .= self::EncodeTuple( $indent + 1, $tuple );
                $sep = ",";
            }
            
            $json .= "]";
        }
        
        $json .= "}";

        return $json;
    }

    static function EncodeTuple( $indent, $tuple )
    {
        $json = "\n" . str_repeat( " ", $indent * 4 );
        $json .= "{";
        $sep = "";
    
        foreach ( $tuple as $key => $value )
        {
            $json .= $sep;

            $json .= self::EncodeStringValue( $indent + 1, $key, $value );

            $sep = ",";
        }

        $json .= "\n" . str_repeat( " ", $indent * 4 ) . "}";

        return $json;
    }

    static function Encode( $indent, $something )
    {
        $json = "" . str_repeat( " ", $indent * 4 );

        if ( is_array( $something ) )
        {
            $json .= self::EncodeArray( $indent + 1, $something );
        }
        else
        {
            $json .= self::EncodeObject( $indent + 1, $something );
        }

        return $json;
    }

    static function EncodeArray( $indent, $array )
    {
        $json = "\n" . str_repeat( " ", $indent * 4 );
        $json .= "[";
        $sep = "";
        
        if ( self::is_assoc( $array ) )
        {
            foreach ( $array as $string => $value )
            {
                $json .= $sep . "{";
                
                $json .= self::EncodeStringValue( $indent + 1, $string, $value );
                
                $json .= "}";

                $sep = ",";
            }
        }
        else
        {
            foreach ( $array as $value )
            {
                $json .= $sep;
                
                $json .= self::EncodeValue( $indent + 1, $value );
                
                $sep = ",";
            }
        }
        $json .= "\n" . str_repeat( " ", $indent * 4 ) . "]";

        return $json;
    }

    static function EncodeObject( $indent, $object )
    {
        $json = "\n" . str_repeat( " ", $indent * 4 );
        $json .= "{";
        $sep = "";

        foreach ( $object as $member => $value )
        {
            $json .= $sep;
            
            $json .= self::EncodeStringValue( $indent + 1, $member, $value );

            $sep = ",";
        }
        $json .= "\n" . str_repeat( " ", $indent * 4 ) . "}";

        return $json;
    }

    static function EncodeStringValue( $indent, $string, $value )
    {
        $json = "\n" . str_repeat( " ", $indent * 4 );
        $json .= self::EncodeString( $indent + 1, $string );
        $json .= ":";
        $json .= self::EncodeValue( $indent + 1, $value );

        return $json;
    }

    static function EncodeString( $indent, $string )
    {
        $json = "";
        $escaped = str_replace( "\\", "\\\\", $string );

        $json .= "\"$escaped\"";

        return $json;
    }

    static function EncodeValue( $indent, $value )
    {
        $json = "";
        if ( is_array( $value ) )
        {
            $json .= self::EncodeArray( $indent, $value );
        }
        else
        if ( is_string( $value ) )
        {
            $json .= self::EncodeString( $indent, $value );
        }
        else
        if ( is_numeric( $value ) )
        {
            $json .= self::EncodeNumber( $indent, $value );
        }
        else
        if ( is_null( $value ) )
        {
            $json .= "null";
        }
        else
        if ( true === $value )
        {
            $json .= "true";
        }
        else
        if ( false === $value )
        {
            $json .= "false";
        }
        else
        {
            $json .= self::EncodeObject( $indent, $value );
        }

        return $json;
    }

    static function EncodeNumber( $indent, $number )
    {
        $json = "";
        $json .= $number;

        return $json;
    }

    static function is_assoc( $a )
    {
        $assoc = true;
    
        $keys = array_keys( $a );
        foreach ( $keys as $key )
        {
            if ( is_numeric( $key ) && (0 == $key) ) $assoc = false;
            break;
        }
        return $assoc;
    }

    static function is_assocx( $a )
    {
        $b = array_keys($a);

        return ($a != array_keys($b));
    }
}

